<?php

namespace App\Http\Controllers\Api\V1;

use App\Http\Controllers\Controller;
use App\Http\Requests\Api\FeedbackFormRequest;
use App\Http\Requests\Api\SubmitFeedbackFormRequest;
use App\Http\Resources\FeedbackFormResource;
use App\Models\FeedbackForm;
use Illuminate\Http\JsonResponse;

class FeedbackFormController extends Controller
{
    protected array $relations = ["company", "questions", "answers", "employees"];

    /**
     * Display a listing of the feedback forms.
     */
    public function index(): JsonResponse {
        $feedbackForms = FeedbackForm::query()->latest()->get();

        return $this->respondWithResourceCollection(
            FeedbackFormResource::collection($feedbackForms)
        );
    }

    /**
     * Store a newly created feedback form in storage.
     */
    public function store(FeedbackFormRequest $request): JsonResponse {
        $feedbackForm = FeedbackForm::query()->create($request->validated());

        // create questions if provided
        $request->whenFilled("questions",
            fn($questions) => $feedbackForm->questions()->createMany($questions));

        return $this->respondCreated(
            new FeedbackFormResource($feedbackForm),
            "Feedback form created successfully!"
        );
    }

    /**
     * Display the specified feedback form.
     */
    public function show(FeedbackForm $feedbackForm): JsonResponse {
        $feedbackForm->loadMissing($this->relations);

        return $this->respondWithResource(
            new FeedbackFormResource($feedbackForm)
        );
    }

    /**
     * Update the specified feedback form in storage.
     */
    public function update(FeedbackFormRequest $request, FeedbackForm $feedbackForm): JsonResponse {
        $feedbackForm->update($request->validated());

        // execute function if questions are provided
        $request->whenFilled("questions", function ($questions) use ($feedbackForm) {
            // update or create questions
            $reqQuestionIds = collect($questions)->map(
                fn($question) => $feedbackForm->questions()->updateOrCreate(
                    ["id" => $question["id"] ?? null], $question)->id)->toArray();

            // delete questions that aren't included in the request
            $feedbackForm->questions()->whereNotIn("id", $reqQuestionIds)->delete();
        });

        $feedbackForm->loadMissing(["questions"]);

        return $this->respondUpdated(
            new FeedbackFormResource($feedbackForm),
            "Feedback form updated successfully!"
        );
    }

    /**
     * Remove the specified feedback form from storage.
     */
    public function destroy(FeedbackForm $feedbackForm): JsonResponse {
        $feedbackForm->delete();

        return $this->respondSuccess(
            "Feedback form deleted successfully!"
        );
    }

    public function submit(SubmitFeedbackFormRequest $request, FeedbackForm $feedbackForm): JsonResponse {
        $userId = auth()->user()->getAuthIdentifier();

        // Check if the user has already submitted feedback for this form
        if ($feedbackForm->employees()->where('user_id', $userId)->exists()) {
            return $this->respondError("You have already submitted feedback for this form.", null, 403);
        }

        $answers = collect($request->validated("answers"))->map(
            fn($answer) => ["user_id" => $userId, ...$answer])->toArray();

        $feedbackForm->answers()->createMany($answers);

        // Associate the user with this feedback form
        $feedbackForm->employees()->attach($userId, ['submitted_at' => now()]);

        return $this->respondSuccess(
            "Feedback form submitted successfully!"
        );
    }
}
