<?php

namespace App\Http\Controllers\Api\V1;

use App\Http\Controllers\Controller;
use App\Http\Requests\Api\InvoiceRequest;
use App\Http\Resources\InvoiceResource;
use App\Models\Invoice;
use Illuminate\Http\JsonResponse;

class InvoiceController extends Controller
{
    protected array $fileFieldNames = ["file"];
    protected array $fileFolderPaths = ["invoices"];
    protected array $relations = ["company", "client"];

    /**
     * Display a listing of the invoices.
     */
    public function index(): JsonResponse {
        $page = request()->query("page");

        $invoices = Invoice::filter()
            ->latest()
            ->when($page,
                fn($query) => $query->paginate(request()->query("perPage")),
                fn($query) => $query->get()
            );

        return $this->respondWithResourceCollection(
            InvoiceResource::collection($invoices)
        );
    }

    /**
     * Store a newly created invoice in storage.
     */
    public function store(InvoiceRequest $request): JsonResponse {
        $invoice = new Invoice($request->validated());
        $this->storeFiles($request, $invoice);
        $invoice->save();

        // create provided invoice details and link to current invoice
        $request->whenFilled("invoice_details",
            fn($invoiceDetails) => $invoice->invoiceDetails()->createMany($invoiceDetails));

        return $this->respondCreated(
            new InvoiceResource(Invoice::query()->create($request->validated())),
            "Invoice created successfully!"
        );
    }

    /**
     * Display the specified invoice.
     */
    public function show(Invoice $invoice): JsonResponse {
        return $this->respondWithResource(
            new InvoiceResource($invoice)
        );
    }

    /**
     * Update the specified invoice in storage.
     */
    public function update(InvoiceRequest $request, Invoice $invoice): JsonResponse {
        $invoice->fill($request->validated());
        $this->updateFiles($request, $invoice);
        $invoice->save();

        $request->whenFilled("invoice_details", function ($invoiceDetails) use ($invoice) {
            // update or create provided invoice details
            $reqInvoiceDetailIds = collect($invoiceDetails)->map(
                fn($detail) => $invoice->invoiceDetails()->updateOrCreate(
                    ["id" => $detail["id"] ?? null], $detail)->id)->toArray();

            // delete invoice details that aren't included in the request
            $invoice->invoiceDetails()
                ->whereNotIn("id", $reqInvoiceDetailIds)->delete();
        });

        return $this->respondUpdated(
            new InvoiceResource($invoice),
            "Invoice updated successfully!"
        );
    }

    /**
     * Remove the specified invoice from storage.
     */
    public function destroy(Invoice $invoice): JsonResponse {
        // $this->deleteFiles($invoice);
        $invoice->delete();

        return $this->respondSuccess(
            "Invoice deleted successfully"
        );
    }
}
