<?php

namespace App\Http\Controllers\Api\V1;

use App\Http\Controllers\Controller;
use App\Http\Requests\Api\IssueRequest;
use App\Http\Resources\IssueResource;
use App\Models\Issue;
use Illuminate\Http\JsonResponse;

class IssueController extends Controller
{
    protected array $fileFieldNames = ["file"];
    protected array $fileFolderPaths = ["issues"];
    protected array $relations = ["company", "project", "user"];

    /**
     * Display a listing of the issues.
     */
    public function index(): JsonResponse {
        $page = request()->query("page");

        $issues = Issue::filter()
            ->latest()
            ->when($page,
                fn($query) => $query->paginate(request()->query("perPage")),
                fn($query) => $query->get()
            );

        return $this->respondWithResourceCollection(
            IssueResource::collection($issues),
        );
    }

    /**
     * Store a newly created issue in storage.
     */
    public function store(IssueRequest $request): JsonResponse {
        $issue = new Issue($request->validated());
        $this->storeFiles($request, $issue);
        $issue->save();

        return $this->respondCreated(
            new IssueResource($issue),
            "Issue created successfully!"
        );
    }

    /**
     * Display the specified issue.
     */
    public function show(Issue $issue): JsonResponse {
        $issue->loadMissing($this->relations);

        return $this->respondWithResource(
            new IssueResource($issue),
        );
    }

    /**
     * Update the specified issue in storage.
     */
    public function update(IssueRequest $request, Issue $issue): JsonResponse {
        $issue->fill($request->validated());
        $this->updateFiles($request, $issue);
        $issue->save();

        return $this->respondUpdated(
            new IssueResource($issue),
            "Issue updated successfully!"
        );
    }

    /**
     * Remove the specified issue from storage.
     */
    public function destroy(Issue $issue): JsonResponse {
        // $this->deleteFiles($issue);
        $issue->delete();

        return $this->respondSuccess(
            "Issue deleted successfully!"
        );
    }
}
