<?php

namespace App\Http\Controllers\Api\V1;

use App\Events\MeetingScheduled;
use App\Http\Controllers\Controller;
use App\Http\Requests\Api\MeetingRequest;
use App\Http\Resources\MeetingResource;
use App\Models\Meeting;
use Illuminate\Http\JsonResponse;

class MeetingController extends Controller
{
    protected array $relations = ["company", "client", "users"];

    /**
     * Display a listing of the meetings.
     */
    public function index(): JsonResponse {
        $page = request()->query("page");

        $meetings = Meeting::filter()
            ->latest()
            ->when($page,
                fn($query) => $query->paginate(request()->query("perPage")),
                fn($query) => $query->get()
            );

        return $this->respondWithResourceCollection(
            MeetingResource::collection($meetings)
        );
    }

    /**
     * Store a newly created meeting in storage.
     */
    public function store(MeetingRequest $request): JsonResponse {
        $meeting = Meeting::query()->create($request->validated());

        // attach users if user_ids are provided (meeting_user pivot table)
        $request->whenFilled("user_ids",
            fn($userIds) => $meeting->users()->attach($userIds));

        // dispatch meeting scheduled event
        event(new MeetingScheduled($meeting));

        return $this->respondCreated(
            new MeetingResource($meeting),
            "Meeting created successfully!"
        );
    }

    /**
     * Display the specified meeting.
     */
    public function show(Meeting $meeting): JsonResponse {
        $meeting->loadMissing($this->relations);

        return $this->respondWithResource(
            new MeetingResource($meeting)
        );
    }

    /**
     * Update the specified meeting in storage.
     */
    public function update(MeetingRequest $request, Meeting $meeting): JsonResponse {
        $meeting->update($request->validated());

        // sync users if user_ids are provided
        $request->whenFilled("user_ids",
            fn($userIds) => $meeting->users()->sync($userIds));


        return $this->respondCreated(
            new MeetingResource($meeting),
            "Meeting updated successfully!"
        );
    }

    /**
     * Remove the specified meeting from storage.
     */
    public function destroy(Meeting $meeting): JsonResponse {
        $meeting->delete();

        return $this->respondSuccess(
            "Meeting deleted successfully"
        );
    }
}
