<?php

namespace App\Http\Controllers\Api\V1;

use App\Http\Controllers\Controller;
use App\Http\Requests\Api\ManyPermissionsRequest;
use App\Http\Requests\Api\PermissionRequest;
use App\Http\Resources\PermissionResource;
use App\Models\Permission;
use App\Models\User;
use Illuminate\Http\JsonResponse;

class PermissionController extends Controller
{
    protected array $relations = ["user", "page"];

    /**
     * Display a listing of the permissions.
     */
    public function index(User $user): JsonResponse {
        $page = request()->query("page");

        $permissions = $user->permissions()
            ->filter()
            ->latest()
            ->when($page,
                fn($query) => $query->paginate(request()->query("perPage")),
                fn($query) => $query->get()
            );

        return $this->respondWithResourceCollection(
            PermissionResource::collection($permissions),
        );
    }

    /**
     * Store a newly created permission in storage.
     */
    public function store(PermissionRequest $request, User $user): JsonResponse {
        $permission = $user->permissions()->create($request->validated());

        return $this->respondCreated(
            new PermissionResource($permission),
            "Permission created successfully!"
        );
    }

    /**
     * Display the specified permission.
     */
    public function show(User $user, Permission $permission): JsonResponse {
        $permission->loadMissing($this->relations);

        return $this->respondWithResource(
            new PermissionResource($permission)
        );
    }

    /**
     * Update the specified permission in storage.
     */
    public function updateMany(ManyPermissionsRequest $request, User $user): JsonResponse {
        $permissions = collect($request->input("permissions"))->map(function ($permissionData) use ($user) {
            $permission = $user->permissions()->findOrFail($permissionData["id"]);
            $permission->update($permissionData);
            return $permission;
        });

        return $this->respondUpdated(
            PermissionResource::collection($permissions),
            "Permission updated successfully!"
        );
    }

    /**
     * Update the specified permission in storage.
     */
    public function update(PermissionRequest $request, User $user, Permission $permission): JsonResponse {
        $permission->update($request->validated());

        return $this->respondUpdated(
            new PermissionResource($permission),
            "Permission updated successfully!"
        );
    }

    /**
     * Remove the specified permission from storage.
     */
    public function destroy(User $user, Permission $permission): JsonResponse {
        $user->permissions()->findOrFail($permission->id)->delete();

        return $this->respondSuccess(
            "Permission deleted successfully!"
        );
    }
}
