<?php

namespace App\Http\Controllers\Api\V1;

use App\Http\Controllers\Controller;
use App\Http\Requests\Api\ProjectRequest;
use App\Http\Resources\ProjectResource;
use App\Models\Project;
use Illuminate\Http\JsonResponse;

class ProjectController extends Controller
{
    protected array $relations = [
        "company", "service", "client", "invoice", "projectResources", "tasks",
        "users", "credentials", "issues", "taskLogs"
    ];

    /**
     * Display a listing of the projects.
     */
    public function index(): JsonResponse {
        $page = request()->query("page");

        $projects = Project::filter()
            ->latest()
            ->when($page,
                fn($query) => $query->paginate(request()->query("perPage")),
                fn($query) => $query->get()
            );

        return $this->respondWithResourceCollection(
            ProjectResource::collection($projects),
        );
    }

    /**
     * Store a newly created project in storage.
     */
    public function store(ProjectRequest $request): JsonResponse {
        $project = Project::query()->create($request->validated());

        // attach users if user_ids are provided (project_user pivot table)
        $request->whenFilled("user_ids",
            fn($userIds) => $project->users()->attach($userIds));

        // create project resources if provided
        $request->whenFilled("project_resources",
            fn($projectResources) => $project->projectResources()->createMany($projectResources));

        return $this->respondCreated(
            new ProjectResource($project),
            "Project created successfully!",
        );
    }

    /**
     * Display the specified project.
     */
    public function show(Project $project): JsonResponse {
        $project->loadMissing($this->relations);
        return $this->respondWithResource(
            new ProjectResource($project),
        );
    }

    /**
     * Update the specified project in storage.
     */
    public function update(ProjectRequest $request, Project $project): JsonResponse {
        $project->update($request->validated());

        // sync (attach provided and delete other) users if user_ids are provided
        $request->whenFilled("user_ids",
            fn($userIds) => $project->users()->sync($userIds));

        $request->whenFilled("project_resources", function ($projectResources) use ($project) {
            // update or create provided project resources
            $reqResourceIds = collect($projectResources)->map(
                fn($resource) => $project->projectResources()->updateOrCreate(
                    ['id' => $resource['id'] ?? null], $resource)->id)->toArray();

            // delete resources that aren't included in the request
            $project->projectResources()
                ->whereNotIn("id", $reqResourceIds)->delete();
        });

        return $this->respondUpdated(
            new ProjectResource($project),
            "Project updated successfully!"
        );
    }

    /**
     * Remove the specified project from storage.
     */
    public function destroy(Project $project): JsonResponse {
        $project->delete();

        return $this->respondSuccess(
            "Project deleted successfully!"
        );
    }
}
