<?php

namespace App\Http\Controllers\Api\V1;

use App\Http\Controllers\Controller;
use App\Http\Requests\Api\ProjectResourceRequest;
use App\Http\Resources\ProjectResResource;
use App\Models\ProjectResource;
use Illuminate\Http\JsonResponse;

class ProjectResourceController extends Controller
{
    protected array $fileFieldNames = ["resource_document", "build", "apk"];
    protected array $fileFolderPaths = ["resource-documents", "builds", "apks"];
    protected array $relations = ["project"];

    /**
     * Display a listing of the project resources.
     */
    public function index(): JsonResponse {
        $page = request()->query("page");

        $projectResources = ProjectResource::filter()
            ->latest()
            ->when($page,
                fn($query) => $query->paginate(request()->query("perPage")),
                fn($query) => $query->get()
            );

        return $this->respondWithResourceCollection(
            ProjectResResource::collection($projectResources),
        );
    }

    /**
     * Store a newly created project resource in storage.
     */
    public function store(ProjectResourceRequest $request): JsonResponse {
        $projectResource = new ProjectResource($request->validated());
        $this->storeFiles($request, $projectResource);
        $projectResource->save();

        return $this->respondCreated(
            new ProjectResResource($projectResource),
            "Project resource created successfully!"
        );
    }

    /**
     * Display the specified project resource.
     */
    public function show(ProjectResource $projectResource): JsonResponse {
        $projectResource->loadMissing($this->relations);
        return $this->respondWithResource(
            new ProjectResResource($projectResource),
        );
    }

    /**
     * Update the specified project resource in storage.
     */
    public function update(ProjectResourceRequest $request, ProjectResource $projectResource): JsonResponse {
        $projectResource->fill($request->validated());
        $this->updateFiles($request, $projectResource);
        $projectResource->save();

        return $this->respondUpdated(
            new ProjectResResource($projectResource),
            "Project resource updated successfully!"
        );
    }

    /**
     * Remove the specified project resource from storage.
     */
    public function destroy(ProjectResource $projectResource): JsonResponse {
        // $this->deleteFiles($projectResource);
        $projectResource->delete();

        return $this->respondSuccess(
            "Project resource deleted successfully!"
        );
    }
}
