<?php

namespace App\Http\Controllers\Api\V1;

use App\Http\Controllers\Controller;
use App\Http\Requests\Api\TaskRequest;
use App\Http\Resources\TaskResource;
use App\Models\Task;
use Illuminate\Http\JsonResponse;

class TaskController extends Controller
{
    protected array $relations = ["company", "project", "users", "amcTask"];

    /**
     * Display a listing of the tasks.
     */
    public function index(): JsonResponse {
        $page = request()->query("page");

        $tasks = Task::filter()
            ->latest()
            ->when($page,
                fn($query) => $query->paginate(request()->query("perPage")),
                fn($query) => $query->get()
            );

        return $this->respondWithResourceCollection(
            TaskResource::collection($tasks)
        );
    }

    /**
     * Store a newly created task in storage.
     */
    public function store(TaskRequest $request): JsonResponse {
        $task = Task::query()->create($request->validated());

        // assign task to users if user_ids are provided
        $request->whenFilled("user_ids",
            fn($userIds) => $task->users()->attach($userIds));

        return $this->respondCreated(
            new TaskResource($task),
            "Task created and users assigned successfully!"
        );
    }

    /**
     * Display the specified task.
     */
    public function show(Task $task): JsonResponse {
        $task->loadMissing($this->relations);

        return $this->respondWithResource(
            new TaskResource($task)
        );
    }

    /**
     * Update the specified task in storage.
     */
    public function update(TaskRequest $request, Task $task): JsonResponse {
        // Update the task details
        $task->update($request->validated());

        // Sync users if user_ids are provided
        $request->whenFilled("user_ids",
            fn($userIds) => $task->users()->sync($userIds));

        return $this->respondUpdated(
            new TaskResource($task),
            "Task updated and users reassigned successfully!"
        );
    }

    /**
     * Remove the specified task from storage.
     */
    public function destroy(Task $task): JsonResponse {
        $task->delete();

        return $this->respondSuccess(
            "Task deleted successfully!"
        );
    }
}
