<?php

namespace App\Http\Traits;

use App\Http\Resources\Ghost\EmptyResource;
use App\Http\Resources\Ghost\EmptyResourceCollection;
use Error;
use Exception;
use Illuminate\Database\QueryException;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Resources\Json\JsonResource;
use Illuminate\Http\Resources\Json\ResourceCollection;
use Illuminate\Validation\ValidationException;

trait HasApiResponse
{

    public function respondCreated($data, $message = "Resource created successfully"): JsonResponse {
        return $this->respondSuccess($message, $data, 201);
    }

    public function respondSuccess(
        string $message = null,
        mixed $data = null,
        int $statusCode = 200,
        array $headers = []
    ): JsonResponse {
        return $this->apiResponse([
            "success" => true,
            "message" => $message,
            "data" => $data
        ], $statusCode, $headers);
    }

    public function apiResponse(array $data, int $statusCode, array $headers = []): JsonResponse {
        $data = $this->parseGivenData($data, $statusCode, $headers);

        return response()->json(
            $data['content'], $data['statusCode'], $data['headers']
        );
    }

    public function parseGivenData($data = [], $statusCode = 200, $headers = []): array {
        $responseStructure = [
            'success' => $data['success'],
//            'message' => $data['message'] ?? null,
        ];

        isset($data["message"]) && $responseStructure["message"] = $data["message"];
        isset($data["data"]) && $responseStructure["data"] = $data["data"];
        isset($data["links"]) && $responseStructure["links"] = $data["links"];
        isset($data["meta"]) && $responseStructure["meta"] = $data["meta"];
        isset($data["token"]) && $responseStructure["token"] = $data["token"];
        isset($data["errors"]) && $responseStructure["errors"] = $data["errors"];

        if (
            isset($data['exception']) &&
            ($data['exception'] instanceof Error || $data['exception'] instanceof Exception)) {

            if (config('app.env') !== 'production') {
                $exception = [
                    'message' => $data['exception']->getMessage(),
                    'file' => $data['exception']->getFile(),
                    'line' => $data['exception']->getLine(),
                    'code' => $data['exception']->getCode(),
                ];

                if ($data["exception"] instanceof QueryException) {
                    $exception["sql"] = $data["exception"]->getSql();
                    $exception['bindings'] = $data["exception"]->getBindings();
                }

                if ($data["exception"] instanceof ValidationException) {
                    $exception = [];
                }

                $responseStructure["exception"] = $exception;
            }

            $statusCode === 200 && $statusCode = 500;
        }

        return ["content" => $responseStructure, "statusCode" => $statusCode, "headers" => $headers];
    }

    public function respondQueryException(
        QueryException $exception,
        string $message = "There was an issue with the database query",
        int $statusCode = 500,
    ): JsonResponse {
        return $this->apiResponse([
            "success" => false,
            "message" => $message,
            "exception" => $exception
        ], $statusCode);
    }

    public function respondUpdated($data, $message = "Resource updated successfully"): JsonResponse {
        return $this->respondSuccess($message, $data);
    }

    public function respondAuthenticated(
        string $token,
        $message,
        mixed $data,
        $statusCode = 200,
        $headers = []
    ): JsonResponse {
        return $this->apiResponse([
            "success" => true,
            "message" => $message,
            "token" => $token,
            "data" => $data,
        ], $statusCode, $headers);
    }

    public function respondNoContent($message = "No Content Found"): JsonResponse {
        return $this->respondSuccess($message);
    }

    public function respondNoContentResource($message = "No Content Found"): JsonResponse {
        return $this->respondWithResource(new EmptyResource([]), $message);
    }

    public function respondWithResource(
        JsonResource $resource,
        $statusCode = 200,
        $headers = []
    ): JsonResponse {
        return $this->respondSuccess(null, $resource, $statusCode, $headers);
    }

    public function respondNoContentResourceCollection($message = 'No Content Found'): JsonResponse {
        return $this->respondWithResourceCollection(new EmptyResourceCollection([]), $message);
    }

    public function respondWithResourceCollection(
        ResourceCollection $resourceCollection,
        $statusCode = 200,
        $headers = []
    ): JsonResponse {
        $collection = $resourceCollection->response()->getData();

        return
            $this->apiResponse([
                "success" => true,
                "message" => null,
                "data" => $collection->data,
                "links" => $collection?->links ?? null,
                "meta" => $collection?->meta ?? null
            ],
                $statusCode,
                $headers
            );
    }

    public function respondNotFound($message = "Resource Not Found", $exception = null): JsonResponse {
        return $this->respondError($message, $exception, 404);
    }

    public function respondError(
        $message = "There was an internal error, Pls try again later",
        Exception|null $exception = null,
        int $statusCode = 500,
    ): JsonResponse {
        return $this->apiResponse(
            [
                "success" => false,
                "message" => $message,
                "exception" => $exception
            ], $statusCode
        );
    }

    public function respondValidationErrors(ValidationException $exception): JsonResponse {
        return $this->apiResponse(
            [
                'success' => false,
                'message' => $exception->getMessage(),
                'errors' => $exception->errors(),
                'exception' => $exception,
            ],
            422
        );
    }

    public function respondUnAuthorized($message = "Unauthorized"): JsonResponse {
        return $this->respondError($message, statusCode: 401);
    }

    public function respondForbidden($message = "Forbidden"): JsonResponse {
        return $this->respondError($message, statusCode: 403);
    }
}


