<?php

namespace Database\Seeders;

use App\Models\Attendance;
use App\Models\Client;
use App\Models\Command;
use App\Models\Company;
use App\Models\Complaint;
use App\Models\Credential;
use App\Models\EmployeeBreak;
use App\Models\Expense;
use App\Models\ExpenseType;
use App\Models\FollowUp;
use App\Models\Holiday;
use App\Models\Invoice;
use App\Models\InvoiceDetail;
use App\Models\Issue;
use App\Models\Lead;
use App\Models\Leave;
use App\Models\Meeting;
use App\Models\Notice;
use App\Models\Notification;
use App\Models\Project;
use App\Models\Proposal;
use App\Models\Rating;
use App\Models\Role;
use App\Models\Service;
use App\Models\Task;
use App\Models\User;
use App\Models\Vendor;
use App\Models\VendorTransaction;
use Illuminate\Database\Seeder;

// use Illuminate\Database\Console\Seeds\WithoutModelEvents;

class DatabaseSeeder extends Seeder
{
    /**
     * Seed the application's database.
     */
    public function run(): void {
        // User::factory(10)->create();

        $company = Company::factory()->create([
            "name" => "UD IT Solutions",
            "founder_name" => "Udit Takwani",
        ]);

        $serviceNames = ["Web Dev", "App Dev", "SEO"];

        // create services, clients, projects and tasks
        foreach ($serviceNames as $i => $name) {
            $service = Service::factory()->create([
                "company_id" => $company->id,
                "name" => $name,
            ]);

            $companyServiceIds = [
                "company_id" => $company->id,
                "service_id" => $service->id,
            ];

            $lead = Lead::factory()->create($companyServiceIds);
            Proposal::factory()->create($companyServiceIds);

            FollowUp::factory()->create([
                "lead_id" => $lead->id,
            ]);

            $vendor = Vendor::factory()->create($companyServiceIds);

            VendorTransaction::factory()->create([
                "vendor_id" => $vendor->id,
            ]);

            $expenseType = ExpenseType::factory()->create();

            Expense::factory()->create([
                "company_id" => $company->id,
                "expense_type_id" => $expenseType->id,
                "vendor_id" => $vendor->id,
            ]);

            $client = Client::factory()->create([
                "company_id" => $company->id,
            ]);

            $project = Project::factory()->create([
                "company_id" => $company->id,
                "service_id" => $service->id,
                "client_id" => $client->id,
                "name" => "Project ".$i + 1,
            ]);

            Credential::factory()->create([
                "project_id" => $project->id,
                "client_id" => $client->id,
            ]);

            Task::factory()->create([
                "name" => "Task".$i + 1,
                "company_id" => $company->id,
                "project_id" => $project->id,
            ]);

            $amount = random_int(50_000, 5_00_000);
            $invoice = Invoice::factory()->create([
                "company_id" => $company->id,
                "client_id" => $client->id,
                "amount" => $amount,
            ]);

            InvoiceDetail::factory()->create([
                "invoice_id" => $invoice->id,
                "service_id" => $service->id,
                "amount" => $amount,
            ]);
        }

        // create user roles
        $roleNames = ["admin", "hr", "employee"];
        foreach ($roleNames as $name) {
            Role::factory()->create(["name" => $name]);
        }

        // create user admin
        $admin = User::factory()->create([
            "name" => "Udit Takwani",
            "email" => "admin@udit-solutions.com",
            "role_id" => Role::IS_ADMIN,
            "is_admin" => true,
        ]);

        // create user hr
        $hr = User::factory()->create([
            "company_id" => $company->id,
            "name" => "Sandeep",
            "email" => "sandeep@udit-solutions.com",
            "role_id" => Role::IS_HR,
        ]);


        // create an attendance for hr
        Attendance::factory()->create(["user_id" => $hr->id]);

        // create a notification for hr
        Notification::factory()->create([
            "company_id" => $company->id,
            "user_id" => $hr->id,
        ]);

        // create a leave for hr
        Leave::factory()->create(["user_id" => $hr->id]);

        // create complaint for hr
        Complaint::factory()->create([
            "company_id" => $company->id,
            "user_id" => $hr->id,
        ]);

        $projects = Project::all();
        $employeeNames = ["Tarachand", "Akshay", "Sujeet", "Shubham", "Raj", "Pradeep"];

        // create user employees
        foreach ($employeeNames as $name) {
            $user = User::factory()->create([
                "company_id" => $company->id,
                "name" => $name,
                "email" => strtolower($name)."@udit-solutions.com",
                "role_id" => Role::IS_EMPLOYEE
            ]);

            Issue::factory()->create([
                "company_id" => $company->id,
                "project_id" => $projects->random(1)->first()->id,
                "user_id" => $user->id,
            ]);
        }

        $employees = User::query()->where("role_id", 3)->get();

        // ratings
        foreach ($employees as $employee) {
            // rating the hr by each employee
            Rating::factory()->create([
                "user_id" => $hr->id,
                "rater_id" => $employee->id,
            ]);

            // rating each employee by the hr
            Rating::factory()->create([
                "user_id" => $employee->id,
                "rater_id" => $hr->id,
            ]);

            // rating each employee by all the other employees
            $raters = $employees->filter(fn($rater) => $rater->id !== $employee->id);
            foreach ($raters as $rater) {
                Rating::factory()->create([
                    "user_id" => $employee->id,
                    "rater_id" => $rater->id,
                ]);
            }

            // create attendances
            Attendance::factory()->create(
                ["user_id" => $employee->id]
            );

            // create employeeBreaks for each employee
            EmployeeBreak::factory()->create(["user_id" => $employee->id]);

            // create notification for each employee
            Notification::factory()->create([
                "company_id" => $company->id,
                "user_id" => $employee->id,
            ]);

            // create leave for each employee
            Leave::factory()->create(["user_id" => $employee->id]);

            // create complaint for each employee
            Complaint::factory()->create([
                "company_id" => $company->id,
                "user_id" => $employee->id,
            ]);
        }

        // create a meeting
        Meeting::factory()->create(["company_id" => $company->id]);

        // create a notice
        Notice::factory()->create(["company_id" => $company->id]);

        // create a holiday
        Holiday::factory()->create();
    }
}
